@@ -16,7 +16,15 @@ from asyncio import (
     create_task,
     sleep,
 )
-from collections.abc import Callable, Hashable, Iterable, Iterator
+from collections.abc import (
+    Callable,
+    Hashable,
+    ItemsView,
+    Iterable,
+    Iterator,
+    KeysView,
+    ValuesView,
+)
 from contextlib import (
     AbstractAsyncContextManager,
     AsyncExitStack,
@@ -30,7 +38,16 @@ from itertools import chain
 from logging import DEBUG, Logger, getLogger
 from subprocess import PIPE
 from sys import stderr, stdout
-from typing import TYPE_CHECKING, Any, Self, TextIO, assert_never, overload, override
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    ClassVar,
+    Self,
+    TextIO,
+    assert_never,
+    overload,
+    override,
+)
 
 from typing_extensions import deprecated
 
@@ -39,8 +56,13 @@ from utilities.errors import repr_error
 from utilities.functions import ensure_int, ensure_not_none, to_bool
 from utilities.random import SYSTEM_RANDOM
 from utilities.sentinel import Sentinel, sentinel
-from utilities.types import MaybeCallableBool, SupportsRichComparison
-from utilities.whenever import SECOND, get_now
+from utilities.types import (
+    Delta,
+    MaybeCallableBool,
+    SupportsKeysAndGetItem,
+    SupportsRichComparison,
+)
+from utilities.whenever import SECOND, get_now, round_date_or_date_time, to_nanoseconds
 
 if TYPE_CHECKING:
     from asyncio import _CoroutineLike
@@ -53,7 +75,153 @@ if TYPE_CHECKING:
 
     from whenever import TimeDelta, ZonedDateTime
 
-    from utilities.types import DateTimeRoundUnit, MaybeCallableEvent, MaybeType
+    from utilities.types import MaybeCallableEvent, MaybeType
+
+
+class AsyncDict[K, V]:
+    @overload
+    def __init__(self) -> None: ...
+    @overload
+    def __init__(self, map: SupportsKeysAndGetItem[K, V], /) -> None: ...
+    @overload
+    def __init__(self, iterable: Iterable[tuple[K, V]], /) -> None: ...
+    @override
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
+        super().__init__()
+        self._dict = dict[K, V](*args, **kwargs)
+        self._lock = Lock()
+
+    async def __aenter__(self) -> dict[K, V]:
+        await self._lock.__aenter__()
+        return self._dict
+
+    async def __aexit__(
+        self,
+        exc_type: type[BaseException] | None,
+        exc: BaseException | None,
+        tb: TracebackType | None,
+        /,
+    ) -> None:
+        await self._lock.__aexit__(exc_type, exc, tb)
+
+    def __contains__(self, key: Any, /) -> bool:
+        return key in self._dict
+
+    @override
+    def __eq__(self, other: Any, /) -> bool:
+        return self._dict == other
+
+    __hash__: ClassVar[None] = None  # pyright: ignore[reportIncompatibleMethodOverride]
+
+    def __getitem__(self, key: K, /) -> V:
+        return self._dict[key]
+
+    def __iter__(self) -> Iterator[K]:
+        yield from self._dict
+
+    def __len__(self) -> int:
+        return len(self._dict)
+
+    @override
+    def __repr__(self) -> str:
+        return repr(self._dict)
+
+    def __reversed__(self) -> Iterator[K]:
+        return reversed(self._dict)
+
+    @override
+    def __str__(self) -> str:
+        return str(self._dict)
+
+    @property
+    def empty(self) -> bool:
+        return len(self) == 0
+
+    @classmethod
+    @overload
+    def fromkeys[T](
+        cls, iterable: Iterable[T], value: None = None, /
+    ) -> AsyncDict[T, Any | None]: ...
+    @classmethod
+    @overload
+    def fromkeys[K2, V2](
+        cls, iterable: Iterable[K2], value: V2, /
+    ) -> AsyncDict[K2, V2]: ...
+    @classmethod
+    def fromkeys(
+        cls, iterable: Iterable[Any], value: Any = None, /
+    ) -> AsyncDict[Any, Any]:
+        return cls(dict.fromkeys(iterable, value))
+
+    async def clear(self) -> None:
+        async with self._lock:
+            self._dict.clear()
+
+    def copy(self) -> Self:
+        return type(self)(self._dict.items())
+
+    async def del_(self, key: K, /) -> None:
+        async with self._lock:
+            del self._dict[key]
+
+    @overload
+    def get(self, key: K, default: None = None, /) -> V | None: ...
+    @overload
+    def get(self, key: K, default: V, /) -> V: ...
+    @overload
+    def get[V2](self, key: K, default: V2, /) -> V | V2: ...
+    def get(self, key: K, default: Any = sentinel, /) -> Any:
+        match default:
+            case Sentinel():
+                return self._dict.get(key)
+            case _:
+                return self._dict.get(key, default)
+
+    def keys(self) -> KeysView[K]:
+        return self._dict.keys()
+
+    def items(self) -> ItemsView[K, V]:
+        return self._dict.items()
+
+    @overload
+    async def pop(self, key: K, /) -> V: ...
+    @overload
+    async def pop(self, key: K, default: V, /) -> V: ...
+    @overload
+    async def pop[V2](self, key: K, default: V2, /) -> V | V2: ...
+    async def pop(self, key: K, default: Any = sentinel, /) -> Any:
+        async with self._lock:
+            match default:
+                case Sentinel():
+                    return self._dict.pop(key)
+                case _:
+                    return self._dict.pop(key, default)
+
+    async def popitem(self) -> tuple[K, V]:
+        async with self._lock:
+            return self._dict.popitem()
+
+    async def set(self, key: K, value: V, /) -> None:
+        async with self._lock:
+            self._dict[key] = value
+
+    async def setdefault(self, key: K, default: V, /) -> V:
+        async with self._lock:
+            return self._dict.setdefault(key, default)
+
+    @overload
+    async def update(self, m: SupportsKeysAndGetItem[K, V], /) -> None: ...
+    @overload
+    async def update(self, m: Iterable[tuple[K, V]], /) -> None: ...
+    async def update(self, *args: Any, **kwargs: V) -> None:
+        async with self._lock:
+            self._dict.update(*args, **kwargs)
+
+    def values(self) -> ValuesView[V]:
+        return self._dict.values()
+
+
+##
 
 
 class EnhancedQueue[T](Queue[T]):
@@ -222,7 +390,7 @@ class EnhancedTaskGroup(TaskGroup):
 
     _max_tasks: int | None
     _semaphore: Semaphore | None
-    _timeout: TimeDelta | None
+    _timeout: Delta | None
     _error: MaybeType[BaseException]
     _debug: MaybeCallableBool
     _stack: AsyncExitStack
@@ -233,7 +401,7 @@ class EnhancedTaskGroup(TaskGroup):
         self,
         *,
         max_tasks: int | None = None,
-        timeout: TimeDelta | None = None,
+        timeout: Delta | None = None,
         error: MaybeType[BaseException] = TimeoutError,
         debug: MaybeCallableBool = False,
     ) -> None:
@@ -291,6 +459,23 @@ class EnhancedTaskGroup(TaskGroup):
         _ = self._stack.push_async_callback(cm.__aexit__, None, None, None)
         return self.create_task(cm.__aenter__())
 
+    async def run_or_create_many_tasks[**P, T](
+        self,
+        make_coro: Callable[P, _CoroutineLike[T]],
+        *args: P.args,
+        **kwargs: P.kwargs,
+    ) -> T | Sequence[Task[T]]:
+        match self._is_debug(), self._max_tasks:
+            case (True, _) | (False, None):
+                return await make_coro(*args, **kwargs)
+            case False, int():
+                return [
+                    self.create_task(make_coro(*args, **kwargs))
+                    for _ in range(self._max_tasks)
+                ]
+            case _ as never:
+                assert_never(never)
+
     async def run_or_create_task[T](
         self,
         coro: _CoroutineLike[T],
@@ -975,32 +1160,30 @@ def put_items_nowait[T](items: Iterable[T], queue: Queue[T], /) -> None:
 
 
 async def sleep_max(
-    sleep: TimeDelta | None = None, /, *, random: Random = SYSTEM_RANDOM
+    sleep: Delta | None = None, /, *, random: Random = SYSTEM_RANDOM
 ) -> None:
     """Sleep which accepts deltas."""
     if sleep is None:
         return
-    await asyncio.sleep(random.uniform(0.0, sleep.in_seconds()))
+    await asyncio.sleep(random.uniform(0.0, to_nanoseconds(sleep) / 1e9))
 
 
 ##
 
 
-async def sleep_rounded(
-    *, unit: DateTimeRoundUnit = "second", increment: int = 1
-) -> None:
+async def sleep_rounded(delta: Delta, /) -> None:
     """Sleep until a rounded time."""
-    await sleep_until(get_now().round(unit, increment=increment, mode="ceil"))
+    await sleep_until(round_date_or_date_time(get_now(), delta, mode="ceil"))
 
 
 ##
 
 
-async def sleep_td(delta: TimeDelta | None = None, /) -> None:
+async def sleep_td(delta: Delta | None = None, /) -> None:
     """Sleep which accepts deltas."""
     if delta is None:
         return
-    await sleep(delta.in_seconds())
+    await sleep(to_nanoseconds(delta) / 1e9)
 
 
 ##
@@ -1066,13 +1249,10 @@ async def _stream_one(
 
 @asynccontextmanager
 async def timeout_td(
-    timeout: TimeDelta | None = None,
-    /,
-    *,
-    error: MaybeType[BaseException] = TimeoutError,
+    timeout: Delta | None = None, /, *, error: MaybeType[BaseException] = TimeoutError
 ) -> AsyncIterator[None]:
     """Timeout context manager which accepts deltas."""
-    timeout_use = None if timeout is None else timeout.in_seconds()
+    timeout_use = None if timeout is None else (to_nanoseconds(timeout) / 1e9)
     try:
         async with asyncio.timeout(timeout_use):
             yield
@@ -1081,6 +1261,7 @@ async def timeout_td(
 
 
 __all__ = [
+    "AsyncDict",
     "EnhancedQueue",
     "EnhancedTaskGroup",
     "Looper",
