@@ -71,8 +71,6 @@ from utilities.pathlib import module_path, temp_cwd
 from utilities.platform import IS_WINDOWS
 from utilities.sentinel import Sentinel, sentinel
 from utilities.tempfile import TEMP_DIR, TemporaryDirectory
-from utilities.types import DateTimeRoundUnit
-from utilities.typing import get_literal_elements
 from utilities.version import Version
 from utilities.whenever import (
     DATE_DELTA_MAX,
@@ -88,10 +86,9 @@ from utilities.whenever import (
     DAY,
     TIME_DELTA_MAX,
     TIME_DELTA_MIN,
-    Freq,
     to_date_time_delta,
     to_days,
-    to_nanos,
+    to_nanoseconds,
 )
 from utilities.zoneinfo import UTC, ensure_time_zone
 
@@ -224,10 +221,10 @@ def date_time_deltas(
             ...
         case _ as never:
             assert_never(never)
-    min_nanos, max_nanos = map(to_nanos, [min_value_, max_value_])
+    min_nanos, max_nanos = map(to_nanoseconds, [min_value_, max_value_])
     if draw2(draw, parsable):
-        min_nanos = max(min_nanos, to_nanos(DATE_TIME_DELTA_PARSABLE_MIN))
-        max_nanos = min(max_nanos, to_nanos(DATE_TIME_DELTA_PARSABLE_MAX))
+        min_nanos = max(min_nanos, to_nanoseconds(DATE_TIME_DELTA_PARSABLE_MIN))
+        max_nanos = min(max_nanos, to_nanoseconds(DATE_TIME_DELTA_PARSABLE_MAX))
     nanos = draw(integers(min_value=min_nanos, max_value=max_nanos))
     return to_date_time_delta(nanos)
 
@@ -489,38 +486,6 @@ def floats_extra(
 ##
 
 
-@composite
-def freqs(
-    draw: DrawFn, /, *, unit: MaybeSearchStrategy[DateTimeRoundUnit | None] = None
-) -> Freq:
-    unit_ = draw2(draw, unit, _freq_units())
-    match unit_:
-        case "day":
-            return Freq(unit=unit_)
-        case "hour":
-            return Freq(unit=unit_, increment=draw(_freq_increments(24)))
-        case "minute" | "second":
-            return Freq(unit=unit_, increment=draw(_freq_increments(60)))
-        case "millisecond" | "microsecond" | "nanosecond":
-            return Freq(unit=unit_, increment=draw(_freq_increments(1000)))
-        case _ as never:
-            assert_never(never)
-
-
-@composite
-def _freq_units(draw: DrawFn, /) -> DateTimeRoundUnit:
-    return draw(sampled_from(get_literal_elements(DateTimeRoundUnit)))
-
-
-@composite
-def _freq_increments(draw: DrawFn, n: int, /) -> int:
-    divisors = [i for i in range(1, n) if n % i == 0]
-    return draw(sampled_from(divisors))
-
-
-##
-
-
 @composite
 def git_repos(draw: DrawFn, /) -> Path:
     path = draw(temp_paths())
@@ -815,7 +780,7 @@ def paths(
 @composite
 def _path_parts(draw: DrawFn, /) -> str:
     part = draw(text_ascii(min_size=1, max_size=10))
-    reserved = {"AUX", "NUL"}
+    reserved = {"AUX", "NUL", "nuL"}
     _ = assume(part not in reserved)
     return part
 
@@ -1408,7 +1373,6 @@ __all__ = [
     "float64s",
     "float_arrays",
     "floats_extra",
-    "freqs",
     "git_repos",
     "hashables",
     "import_froms",
