@@ -18,7 +18,7 @@ from itertools import chain
 from math import floor
 from operator import ge, le
 from re import search
-from typing import Any, Literal, TypeGuard, TypeVar, assert_never, cast, override
+from typing import TYPE_CHECKING, Any, Literal, TypeGuard, assert_never, cast, override
 
 from sqlalchemy import (
     URL,
@@ -57,7 +57,8 @@ from sqlalchemy.orm import (
 from sqlalchemy.orm.exc import UnmappedClassError
 from sqlalchemy.pool import NullPool, Pool
 
-from utilities.asyncio import QueueProcessor, timeout_dur
+from utilities.asyncio import Looper, timeout_td
+from utilities.contextlib import suppress_super_object_attribute_error
 from utilities.functions import (
     ensure_str,
     get_class_name,
@@ -65,6 +66,7 @@ from utilities.functions import (
     is_string_mapping,
     is_tuple,
     is_tuple_or_str_mapping,
+    yield_object_attributes,
 )
 from utilities.iterables import (
     CheckLengthError,
@@ -80,9 +82,14 @@ from utilities.iterables import (
 )
 from utilities.reprlib import get_repr
 from utilities.text import snake_case
-from utilities.types import Duration, MaybeIterable, StrMapping, TupleOrStrMapping
+from utilities.types import MaybeIterable, MaybeType, StrMapping, TupleOrStrMapping
+from utilities.whenever import SECOND
+
+if TYPE_CHECKING:
+    from enum import Enum, StrEnum
+
+    from whenever import TimeDelta
 
-_T = TypeVar("_T")
 type _EngineOrConnectionOrAsync = Engine | Connection | AsyncEngine | AsyncConnection
 type Dialect = Literal["mssql", "mysql", "oracle", "postgresql", "sqlite"]
 type ORMInstOrClass = DeclarativeBase | type[DeclarativeBase]
@@ -97,7 +104,7 @@ async def check_engine(
     engine: AsyncEngine,
     /,
     *,
-    timeout: Duration | None = None,
+    timeout: TimeDelta | None = None,
     error: type[Exception] = TimeoutError,
     num_tables: int | tuple[int, float] | None = None,
 ) -> None:
@@ -221,7 +228,7 @@ async def ensure_tables_created(
     engine: AsyncEngine,
     /,
     *tables_or_orms: TableOrORMInstOrClass,
-    timeout: Duration | None = None,
+    timeout: TimeDelta | None = None,
     error: type[Exception] = TimeoutError,
 ) -> None:
     """Ensure a table/set of tables is/are created."""
@@ -250,7 +257,7 @@ async def ensure_tables_created(
 async def ensure_tables_dropped(
     engine: AsyncEngine,
     *tables_or_orms: TableOrORMInstOrClass,
-    timeout: Duration | None = None,
+    timeout: TimeDelta | None = None,
     error: type[Exception] = TimeoutError,
 ) -> None:
     """Ensure a table/set of tables is/are dropped."""
@@ -279,16 +286,34 @@ async def ensure_tables_dropped(
 ##
 
 
+def enum_name(enum: type[Enum], /) -> str:
+    """Get the name of an Enum."""
+    return f"{snake_case(get_class_name(enum))}_enum"
+
+
+##
+
+
+def enum_values(enum: type[StrEnum], /) -> list[str]:
+    """Get the values of a StrEnum."""
+    return [e.value for e in enum]
+
+
+##
+
+
 def get_chunk_size(
     engine_or_conn: _EngineOrConnectionOrAsync,
     /,
     *,
     chunk_size_frac: float = CHUNK_SIZE_FRAC,
-    scaling: float = 1.0,
+    max_length: int = 1,
 ) -> int:
     """Get the maximum chunk size for an engine."""
     max_params = _get_dialect_max_params(engine_or_conn)
-    return max(floor(chunk_size_frac * max_params / scaling), 1)
+    scaling = max(max_length, 1)
+    size = floor(chunk_size_frac * max_params / scaling)
+    return max(size, 1)
 
 
 ##
@@ -376,9 +401,9 @@ async def insert_items(
     snake: bool = False,
     chunk_size_frac: float = CHUNK_SIZE_FRAC,
     assume_tables_exist: bool = False,
-    timeout_create: Duration | None = None,
+    timeout_create: TimeDelta | None = None,
     error_create: type[Exception] = TimeoutError,
-    timeout_insert: Duration | None = None,
+    timeout_insert: TimeDelta | None = None,
     error_insert: type[Exception] = TimeoutError,
 ) -> None:
     """Insert a set of items into a database.
@@ -477,9 +502,9 @@ async def migrate_data(
     table_or_orm_to: TableOrORMInstOrClass | None = None,
     chunk_size_frac: float = CHUNK_SIZE_FRAC,
     assume_tables_exist: bool = False,
-    timeout_create: Duration | None = None,
+    timeout_create: TimeDelta | None = None,
     error_create: type[Exception] = TimeoutError,
-    timeout_insert: Duration | None = None,
+    timeout_insert: TimeDelta | None = None,
     error_insert: type[Exception] = TimeoutError,
 ) -> None:
     """Migrate the contents of a table from one database to another."""
@@ -603,35 +628,30 @@ class TablenameMixin:
 
 
 @dataclass(kw_only=True)
-class Upserter(QueueProcessor[_InsertItem]):
-    """Upsert a set of items into a database."""
+class UpsertService(Looper[_InsertItem]):
+    """Service to upsert items to a database."""
 
+    # base
+    freq: TimeDelta = field(default=SECOND, repr=False)
+    backoff: TimeDelta = field(default=SECOND, repr=False)
+    empty_upon_exit: bool = field(default=True, repr=False)
+    # self
     engine: AsyncEngine
     snake: bool = False
     selected_or_all: _SelectedOrAll = "selected"
     chunk_size_frac: float = CHUNK_SIZE_FRAC
     assume_tables_exist: bool = False
-    timeout_create: Duration | None = None
+    timeout_create: TimeDelta | None = None
     error_create: type[Exception] = TimeoutError
-    timeout_insert: Duration | None = None
+    timeout_insert: TimeDelta | None = None
     error_insert: type[Exception] = TimeoutError
 
-    async def _pre_upsert(self, items: Sequence[_InsertItem], /) -> None:
-        """Pre-upsert coroutine."""
-        _ = items
-
-    async def _post_upsert(self, items: Sequence[_InsertItem], /) -> None:
-        """Post-upsert coroutine."""
-        _ = items
-
     @override
-    async def _process_item(self, item: _InsertItem, /) -> None:
-        """Process the first item."""
-        items = list(chain([item], await self._get_items_nowait()))
-        await self._pre_upsert(items)
+    async def core(self) -> None:
+        await super().core()
         await upsert_items(
             self.engine,
-            *items,
+            *self.get_all_nowait(),
             snake=self.snake,
             selected_or_all=self.selected_or_all,
             chunk_size_frac=self.chunk_size_frac,
@@ -641,7 +661,59 @@ class Upserter(QueueProcessor[_InsertItem]):
             timeout_insert=self.timeout_insert,
             error_insert=self.error_insert,
         )
-        await self._post_upsert(items)
+
+
+@dataclass(kw_only=True)
+class UpsertServiceMixin:
+    """Mix-in for the upsert service."""
+
+    # base - looper
+    upsert_service_freq: TimeDelta = field(default=SECOND, repr=False)
+    upsert_service_backoff: TimeDelta = field(default=SECOND, repr=False)
+    upsert_service_empty_upon_exit: bool = field(default=False, repr=False)
+    upsert_service_logger: str | None = field(default=None, repr=False)
+    upsert_service_timeout: TimeDelta | None = field(default=None, repr=False)
+    upsert_service_debug: bool = field(default=False, repr=False)
+    # base - upsert service
+    upsert_service_database: AsyncEngine
+    upsert_service_snake: bool = False
+    upsert_service_selected_or_all: _SelectedOrAll = "selected"
+    upsert_service_chunk_size_frac: float = CHUNK_SIZE_FRAC
+    upsert_service_assume_tables_exist: bool = False
+    upsert_service_timeout_create: TimeDelta | None = None
+    upsert_service_error_create: type[Exception] = TimeoutError
+    upsert_service_timeout_insert: TimeDelta | None = None
+    upsert_service_error_insert: type[Exception] = TimeoutError
+    # self
+    _upsert_service: UpsertService = field(init=False, repr=False)
+
+    def __post_init__(self) -> None:
+        with suppress_super_object_attribute_error():
+            super().__post_init__()  # pyright: ignore[reportAttributeAccessIssue]
+        self._upsert_service = UpsertService(
+            # looper
+            freq=self.upsert_service_freq,
+            backoff=self.upsert_service_backoff,
+            empty_upon_exit=self.upsert_service_empty_upon_exit,
+            logger=self.upsert_service_logger,
+            timeout=self.upsert_service_timeout,
+            _debug=self.upsert_service_debug,
+            # upsert service
+            engine=self.upsert_service_database,
+            snake=self.upsert_service_snake,
+            selected_or_all=self.upsert_service_selected_or_all,
+            chunk_size_frac=self.upsert_service_chunk_size_frac,
+            assume_tables_exist=self.upsert_service_assume_tables_exist,
+            timeout_create=self.upsert_service_timeout_create,
+            error_create=self.upsert_service_error_create,
+            timeout_insert=self.upsert_service_timeout_insert,
+            error_insert=self.upsert_service_error_insert,
+        )
+
+    def _yield_sub_loopers(self) -> Iterator[Looper[Any]]:
+        with suppress_super_object_attribute_error():
+            yield from super()._yield_sub_loopers()  # pyright: ignore[reportAttributeAccessIssue]
+        yield self._upsert_service
 
 
 ##
@@ -658,9 +730,9 @@ async def upsert_items(
     selected_or_all: _SelectedOrAll = "selected",
     chunk_size_frac: float = CHUNK_SIZE_FRAC,
     assume_tables_exist: bool = False,
-    timeout_create: Duration | None = None,
+    timeout_create: TimeDelta | None = None,
     error_create: type[Exception] = TimeoutError,
-    timeout_insert: Duration | None = None,
+    timeout_insert: TimeDelta | None = None,
     error_insert: type[Exception] = TimeoutError,
 ) -> None:
     """Upsert a set of items into a database.
@@ -780,11 +852,11 @@ async def yield_connection(
     engine: AsyncEngine,
     /,
     *,
-    timeout: Duration | None = None,
-    error: type[Exception] = TimeoutError,
+    timeout: TimeDelta | None = None,
+    error: MaybeType[BaseException] = TimeoutError,
 ) -> AsyncIterator[AsyncConnection]:
     """Yield an async connection."""
-    async with timeout_dur(duration=timeout, error=error), engine.begin() as conn:
+    async with timeout_td(timeout, error=error), engine.begin() as conn:
         yield conn
 
 
@@ -894,9 +966,9 @@ def _is_pair_of_tuple_or_str_mapping_and_table(
     return _is_pair_with_predicate_and_table(obj, is_tuple_or_str_mapping)
 
 
-def _is_pair_with_predicate_and_table(
-    obj: Any, predicate: Callable[[Any], TypeGuard[_T]], /
-) -> TypeGuard[tuple[_T, TableOrORMInstOrClass]]:
+def _is_pair_with_predicate_and_table[T](
+    obj: Any, predicate: Callable[[Any], TypeGuard[T]], /
+) -> TypeGuard[tuple[T, TableOrORMInstOrClass]]:
     """Check if an object is pair and a table."""
     return (
         isinstance(obj, tuple)
@@ -983,21 +1055,31 @@ class _MapMappingToTableSnakeMapNonUniqueError(_MapMappingToTableError):
 
 def _orm_inst_to_dict(obj: DeclarativeBase, /) -> StrMapping:
     """Map an ORM instance to a dictionary."""
-    cls = type(obj)
+    attrs = {
+        k for k, _ in yield_object_attributes(obj, static_type=InstrumentedAttribute)
+    }
+    return {
+        name: _orm_inst_to_dict_one(obj, attrs, name) for name in get_column_names(obj)
+    }
 
-    def is_attr(attr: str, key: str, /) -> str | None:
-        if isinstance(value := getattr(cls, attr), InstrumentedAttribute) and (
-            value.name == key
-        ):
-            return attr
-        return None
 
-    def yield_items() -> Iterator[tuple[str, Any]]:
-        for key in get_column_names(cls):
-            attr = one(attr for attr in dir(cls) if is_attr(attr, key) is not None)
-            yield key, getattr(obj, attr)
+def _orm_inst_to_dict_one(
+    obj: DeclarativeBase, attrs: AbstractSet[str], name: str, /
+) -> Any:
+    attr = one(
+        attr for attr in attrs if _orm_inst_to_dict_predicate(type(obj), attr, name)
+    )
+    return getattr(obj, attr)
 
-    return dict(yield_items())
+
+def _orm_inst_to_dict_predicate(
+    cls: type[DeclarativeBase], attr: str, name: str, /
+) -> bool:
+    cls_attr = getattr(cls, attr)
+    try:
+        return cls_attr.name == name
+    except AttributeError:
+        return False
 
 
 ##
@@ -1037,7 +1119,7 @@ def _prepare_insert_or_upsert_items(
     }
     max_length = max(lengths, default=1)
     chunk_size = get_chunk_size(
-        engine, chunk_size_frac=chunk_size_frac, scaling=max_length
+        engine, chunk_size_frac=chunk_size_frac, max_length=max_length
     )
 
     def yield_pairs() -> Iterator[tuple[Insert, None]]:
@@ -1099,12 +1181,16 @@ __all__ = [
     "InsertItemsError",
     "TablenameMixin",
     "UpsertItemsError",
+    "UpsertService",
+    "UpsertServiceMixin",
     "check_engine",
     "columnwise_max",
     "columnwise_min",
     "create_async_engine",
     "ensure_tables_created",
     "ensure_tables_dropped",
+    "enum_name",
+    "enum_values",
     "get_chunk_size",
     "get_column_names",
     "get_columns",
